library SplitNameByRegExpr;

{$IFDEF WIN64}
  {$E wdx64}
{$ENDIF}
{$IFDEF WIN32}
  {$E wdx}
{$ENDIF}

{$WEAKLINKRTTI ON}

uses
  SysUtils,
  Windows,
  Classes,
  AnsiStrings,
  IniFiles,
  RegularExpressions,
  Math,
  StrUtils,
  contplug;

{$R *.res}

type
  TRegExField = record
    Name: AnsiString;
    RegEx: string;
    UnitsCount: Integer;
    CheckExtension: Integer;
    CheckFullPath: Boolean;
    CaseSensitive: Boolean;
    UseSplit: Boolean;
    Reverse: Boolean;
  end;

var
  RegExFields: array of TRegExField;

const
  DefUnitsCount = 30;
  MaxUnits = 99;
  MaxFields = 99;
  FieldPrefix = 'Field_';
  feFileExt = 1;
  feDirExt  = 2;


function MinMax(AMin, AMax, Value: Integer): Integer; inline;
begin
  if Value < AMin then
    Result := AMin
  else if Value > AMax then
    Result := AMax
  else
    Result := Value;
end;

function CheckSubVal(Value, SubValue: Integer): Boolean; inline;
begin
  Result := (Value and SubValue) = SubValue;
end;


function ContentGetSupportedField(FieldIndex: Integer; FieldName: PAnsiChar;
  Units: PAnsiChar; MaxLen: Integer):integer; stdcall;
var
  UnitsStr: AnsiString;
  I: Integer;
begin
  if FieldIndex < Length(RegExFields) then
  begin
    AnsiStrings.StrLCopy(FieldName, PAnsiChar(RegExFields[FieldIndex].Name), MaxLen - 1);
    for I := 0 to RegExFields[FieldIndex].UnitsCount - 1 do
      UnitsStr := UnitsStr + AnsiString(Format('%.2d', [I])) + '|';
    if Length(UnitsStr) > 0 then
      SetLength(UnitsStr, Length(UnitsStr) - 1);
    AnsiStrings.StrLCopy(Units, PAnsiChar(UnitsStr), MaxLen - 1);
    Result := ft_StringW;
  end else
    Result := ft_NoMoreFields;
end;

function ContentGetValueW(FileName: PWideChar; FieldIndex, UnitIndex: Integer;
  FieldValue: PByte; MaxLen, Flags: Integer): Integer; stdcall;
var
  ResultsArray: array of string;

  procedure AddToArray(const S: string);
  begin
    SetLength(ResultsArray, Length(ResultsArray) + 1);
    ResultsArray[High(ResultsArray)] := S;
  end;

  procedure SwapArray;
  var
    Temp: string;
  begin
    if Length(ResultsArray) > 1 then
      for var I := Low(ResultsArray) to High(ResultsArray) div 2 do
      begin
        Temp := ResultsArray[I];
        ResultsArray[I] := ResultsArray[High(ResultsArray) - I];
        ResultsArray[High(ResultsArray) - I] := Temp;
      end
  end;

var
  FN: string;
  I: Integer;
begin
  if FieldIndex >= Length(RegExFields) then
    Exit(ft_NoSuchField);

  Result := ft_FieldEmpty;

  if RegExFields[FieldIndex].CheckFullPath then
    FN := FileName
  else
    FN := ExtractFileName(FileName);

  if (not CheckSubVal(RegExFields[FieldIndex].CheckExtension, feFileExt)) and
    (not CheckSubVal(RegExFields[FieldIndex].CheckExtension, feDirExt)) then
  begin
    FN := ChangeFileExt(FN, '');
  end else if CheckSubVal(RegExFields[FieldIndex].CheckExtension, feFileExt) and
    CheckSubVal(RegExFields[FieldIndex].CheckExtension, feDirExt) then
  begin
    // nothing to do here now
  end else begin
     var IsFile := FileExists(FileName);
     if (CheckSubVal(RegExFields[FieldIndex].CheckExtension, feDirExt) and IsFile) or
       (CheckSubVal(RegExFields[FieldIndex].CheckExtension, feFileExt) and (not IsFile)) then
       FN := ChangeFileExt(FN, '')
  end;


  try
    var RO: TRegExOptions := [roNotEmpty];
    if not RegExFields[FieldIndex].CaseSensitive then
      Include(RO, roIgnoreCase);

    if RegExFields[FieldIndex].UseSplit then
    begin
      var SA: TArray<string> := TRegEx.Split(FN, RegExFields[FieldIndex].RegEx, RO);
      for I := 0 to High(SA) do
      begin
        if SA[I] <> '' then
          AddToArray(SA[I]);
      end;

    end else begin
      var MC := TRegEx.Matches(FN, RegExFields[FieldIndex].RegEx, RO);
      for I := 0 to MC.Count - 1 do
      begin
        if MC.Item[I].Value <> '' then
          AddToArray(MC.Item[I].Value);
      end;
    end;

  except
    on E: Exception do
    begin
      SetLength(ResultsArray, 1);
      ResultsArray[0] := Format('<REGEX ERROR "%s"> %s: %s', [RegExFields[FieldIndex].RegEx, E.ClassName, E.Message]);
      StrLCopy(PWideChar(FieldValue), PWideChar(ResultsArray[0]), MaxLen div 2 - 2);
      Exit(ft_StringW);
    end;
  end;

  if RegExFields[FieldIndex].Reverse then
    SwapArray;

  //SetLength(NumbersArray, Min(Max(1, RegExFields[FieldIndex].UnitsCount), Length(NumbersArray)));

  if (UnitIndex > -1) and (UnitIndex < Length(ResultsArray)) and
    ((UnitIndex = 0) or (UnitIndex < (RegExFields[FieldIndex].UnitsCount))) then
  begin
    StrLCopy(PWideChar(FieldValue), PWideChar(ResultsArray[UnitIndex]), MaxLen div 2 - 2);
    Result := ft_StringW;
  end;
end;

function ContentGetValue(FileName: PansiChar; FieldIndex, UnitIndex: Integer;
  FieldValue: PByte; MaxLen, Flags: Integer): Integer; stdcall;
begin
  Result := ft_NotSupported;
end;


procedure InitPluginData;
var
  IniName, Section: string;
  INI: TIniFile;
  SL: TStringList;
  I: Integer;
  Field: TRegExField;
begin
  try
    IniName := ChangeFileExt(GetModuleName(HInstance), '.ini');

    SL := TStringList.Create;
    try
      SL.CaseSensitive := False;
      SL.UseLocale := False;
      SL.Duplicates := dupIgnore;

      try
        with TMemIniFile.Create(IniName) do
        try
          ReadSections(SL);
        finally
          Free;
        end;
      except
        // do nothing
      end;

      { Check field names }
      for I := SL.Count - 1 downto 0 do
      begin
        if Pos(FieldPrefix, SL[I]) <> 1 then
        begin
          SL.Delete(I);
          Continue;
        end;
        SL[I] := Trim(Copy(SL[I], Length(FieldPrefix) + 1, MaxInt));
        if SL[I] = '' then
        begin
          SL.Delete(I);
          Continue;
        end;
      end;


      INI := TIniFile.Create(IniName);
      try
        for I := 0 to SL.Count - 1 do
        begin
          if I > MaxFields - 1 then
            Break;
          Section := FieldPrefix + SL[I];
          Field.RegEx := Trim(INI.ReadString(Section, 'RegEx', ''));
          if Field.RegEx = '' then
            Continue;
          Field.CheckExtension := INI.ReadInteger(Section, 'CheckExtension', feDirExt);
          Field.CheckFullPath := INI.ReadBool(Section, 'CheckFullPath', False);
          Field.CaseSensitive := INI.ReadBool(Section, 'CaseSensitive', False);
          Field.UseSplit := INI.ReadBool(Section, 'UseSplit', False);
          Field.UnitsCount := INI.ReadInteger(Section, 'UnitsCount', DefUnitsCount);
          Field.Reverse := INI.ReadBool(Section, 'Reverse', False);

          Field.UnitsCount := MinMax(0, MaxUnits, Field.UnitsCount);
          Field.Name := AnsiString(SL[I]);
          RegExFields := RegExFields + [Field];
        end;
      finally
        INI.Free;
      end;

    finally
      SL.Free;
    end;
  except
    // do nothing
  end;
end;

procedure ContentSetDefaultParams(dps: PContentDefaultParamStruct); stdcall;
begin
  InitPluginData;
end;

exports
  ContentGetSupportedField,
  ContentSetDefaultParams,
  ContentGetValue,
  ContentGetValueW;

procedure DLLEntryProc(EntryCode: integer);
begin
  case EntryCode of
    DLL_PROCESS_ATTACH:
    begin
      //InitPluginData;
    end;
    DLL_PROCESS_DETACH:
    begin
    end;
    DLL_THREAD_ATTACH:
    begin
    end;
    DLL_THREAD_DETACH:
    begin
    end;
  end;
end;

begin
  //ReportMemoryLeaksOnShutdown := True;
  DisableThreadLibraryCalls(HInstance);
  DLLProc := @DLLEntryProc;
  DLLEntryProc(DLL_PROCESS_ATTACH);
end.
